/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.common;

import java.util.HashSet;
import java.util.Set;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.common.AbstractCalculationResult;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.sinfo.collision.CollisionCalcOverviewResult;
import org.dive4elements.river.artifacts.states.DefaultState.ComputeType;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.themes.ThemeDocument;

/**
 * Processor to generate the facet and data series of calculated bed collision counts
 *
 * @author Matthias Schäfer
 *
 */
public final class CollisionCalcProcessor extends AbstractSInfoProcessor {

    public static final String FACET_COLLISION_CALC_COUNT = "sinfo_facet_collision_calc_count";

    public static final String FACET_COLLISION_CALC_COUNT_2 = "sinfo_facet_collision_calc_count.2";

    public static final String FACET_COLLISION_CALC_COUNT_3 = "sinfo_facet_collision_calc_count.3";

    private static final String[] COLLISION_FACETS = new String[] { FACET_COLLISION_CALC_COUNT, FACET_COLLISION_CALC_COUNT_2, FACET_COLLISION_CALC_COUNT_3 };

    private static final String I18N_AXIS_LABEL = "chart.collision_count.section.yaxis.label";

    private static final String I18N_SERIES_NAME_PATTERN = "collision.count.title";

    private static final Set<String> HANDLED_FACET_TYPES = new HashSet<>();

    static {
        HANDLED_FACET_TYPES.add(FACET_COLLISION_CALC_COUNT);
        HANDLED_FACET_TYPES.add(FACET_COLLISION_CALC_COUNT_2);
        HANDLED_FACET_TYPES.add(FACET_COLLISION_CALC_COUNT_3);
    }

    public CollisionCalcProcessor() {
        super(I18N_AXIS_LABEL, HANDLED_FACET_TYPES);
    }

    @Override
    protected String generateSeries(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {

        final int year = getDataIndex(bundle);

        final CollisionCalcOverviewResult result = (CollisionCalcOverviewResult) getResult(generator, bundle);

        final double[][] points = result.getStationPointsByYear(SInfoResultType.collisionCount, year);

        return buildSeriesForType(points, generator, bundle, theme, visible, null);
    }

    public static final Facet createFacet(final CallContext context, final String hash, final String id, final AbstractCalculationResult result,
            final int resultIndex, final int year, final int themeCount) {

        // REMARK: we are using themeCount as facet index, because we get several lines (with the same facet name) for the same
        // result index; however name and index are used on client side as unique id.

        final CollisionCalcOverviewResult ccoResult = (CollisionCalcOverviewResult) result;
        if (ccoResult.getSingleYears() == null) {
            final String facetDescription = Resources.getMsg(context.getMeta(), I18N_SERIES_NAME_PATTERN, I18N_SERIES_NAME_PATTERN, result.getLabel());
            return new SInfoResultFacet(themeCount, resultIndex, -1, FACET_COLLISION_CALC_COUNT, facetDescription, I18N_AXIS_LABEL, ComputeType.ADVANCE, id,
                    hash);
        } else {
            final String facetDescription = Resources.getMsg(context.getMeta(), I18N_SERIES_NAME_PATTERN, I18N_SERIES_NAME_PATTERN, Integer.toString(year));
            return new SInfoResultFacet(themeCount, resultIndex, year, COLLISION_FACETS[themeCount % 3], facetDescription, I18N_AXIS_LABEL, ComputeType.ADVANCE,
                    id, hash);
        }
    }
}